@@ -113,7 +113,7 @@ ancestral.pml <- function(object, type = "marginal", return = "prob") {
   eps <- 1.0e-5
   ind1 <- which(apply(contrast, 1, function(x) sum(x > eps)) == 1L)
   ind2 <- which(contrast[ind1, ] > eps, arr.ind = TRUE)
-  #    pos <- ind2[match(as.integer(1L:ncol(contrast)),  ind2[,2]),1]
+
   pos <- ind2[match(seq_len(ncol(contrast)), ind2[, 2]), 1]
   nco <- as.integer(dim(contrast)[1])
   for (i in 1:l) dat[i, (nTips + 1):m] <- .Call("LogLik2", data, P[i, ], nr, nc,
@@ -165,7 +165,7 @@ ancestral.pml <- function(object, type = "marginal", return = "prob") {
 # }
 
 
-# in ancestral.pml and ancestral.pars
+# in mpr
 ancestral2phyDat <- function(x) {
   eps <- 1.0e-5
   contr <- attr(x, "contrast")
@@ -181,15 +181,6 @@ ancestral2phyDat <- function(x) {
 }
 
 
-# in ancestral.pml
-# variante fuer parsimony und ambiguous DNA, ersetzt durch p2dna
-prob2fitchCoding <- function(x, eps = 0.999) {
-  row_max <- apply(x, 1, max)
-  x <- x / row_max
-  as.vector((x > eps) %*% c(1L, 2L, 4L, 8L))
-}
-
-
 fitchCoding2ambiguous <- function(x, type = "DNA") {
   y <- c(
     1L, 2L, 4L, 8L, 8L, 3L, 5L, 9L, 6L, 10L, 12L, 7L, 11L, 13L,
@@ -210,8 +201,8 @@ fitchCoding2ambiguous2 <- function(x, type = "DNA") {
     "d", "b", "n"
   )
   res <- switch(type,
-    "DNA" = dna[match(x, y)],
-    "RNA" = rna[match(x, y)]
+    "DNA" = dna[fmatch(x, y)],
+    "RNA" = rna[fmatch(x, y)]
   )
   res
 }
@@ -224,70 +215,14 @@ ancestral.pars <- function(tree, data, type = c("MPR", "ACCTRAN"), cost = NULL,
   call <- match.call()
   type <- match.arg(type)
   if (type == "ACCTRAN") {
-    res <- ptree(tree, data, retData = TRUE)[[2]]
+    res <- ptree(tree, data, return = return)
+    attr(res, "call") <- call
   }
-  #    if (type == "MPR")
-  #        res = mpr(tree, data)
   if (type == "MPR") {
     res <- mpr(tree, data, cost = cost, return = return)
     attr(res, "call") <- call
-    return(res)
   }
-  l <- length(tree$tip.label)
-  data_type <- attr(data, "type")
-  x <- attributes(data)
-  m <- dim(res)[2]
-  label <- as.character(1:m)
-  nam <- tree$tip.label
-  label[seq_along(nam)] <- nam
-  x[["names"]] <- label
-
-  nc <- attr(data, "nc")
-  result <- vector("list", m)
-
-  if (data_type == "DNA") {
-    result[1:l] <- subset(data, nam)
-    for (i in (l + 1):m) {
-      result[[i]] <- fitchCoding2ambiguous(res[, i])
-    }
-    # phyDat needs new2old <-
-    # if(return=="prob") tmp <- contrast[tmp, , drop=FALSE]
-  }
-  else {
-    Z <- unique(as.vector(res))
-    tmp <- t(vapply(Z, function(x) dec2bin(x, nc), integer(nc)))
-    tmp <- tmp / rowSums(tmp)
-    dimnames(tmp) <- list(Z, attr(data, "levels"))
-    for (i in seq_len(m)) {
-      result[[i]] <- tmp[as.character(res[, i]), , drop = FALSE]
-      rownames(result[[i]]) <- NULL
-    }
-  }
-  attributes(result) <- x
-  attr(result, "call") <- call
-  if (data_type != "DNA" & return == "phyDat") {
-    contrast <- attr(data, "contrast")
-    # proper format
-    eps <- 1.0e-5
-    ind1 <- which(apply(contrast, 1, function(x) sum(x > eps)) == 1L)
-    ind2 <- which(contrast[ind1, ] > eps, arr.ind = TRUE)
-    #        pos <- ind2[match(as.integer(1L:ncol(contrast)),  ind2[,2]),1]
-    pos <- ind2[match(seq_len(ncol(contrast)), ind2[, 2]), 1]
-    result[1:l] <- subset(data, nam)
-    for (i in (l + 1):length(result)) {
-      tmp <- result[[i]]
-      result[[i]] <- pos[max.col(tmp)]
-    }
-  }
-  if (data_type == "DNA" & return == "prob") {
-    result <- new2old.phyDat(result)
-    for (i in seq_len(length(result))) {
-      tmp <- result[[i]]
-      tmp <- tmp / rowSums(tmp)
-      result[[i]] <- tmp
-    }
-  }
-  result
+  res
 }
 
 
@@ -323,8 +258,8 @@ mpr.help <- function(tree, data, cost = NULL) {
   node <- tree$edge[, 1]
   edge <- tree$edge[, 2]
 
-  node <- as.integer(node - 1)
-  edge <- as.integer(edge - 1)
+  node <- as.integer(node - 1L)
+  edge <- as.integer(edge - 1L)
 
   res <- .Call("sankoffMPR", datf, datp, as.numeric(cost), as.integer(nr),
     as.integer(nc), node, edge,
@@ -355,18 +290,6 @@ mpr <- function(tree, data, cost = NULL, return = "prob") {
   rm <- apply(res[[ntips + 1]], 1, min)
   RM <- matrix(rm, nr, nc) + eps
 
-  #    if(return!="prob" & type=="DNA"){
-  #        for(i in (ntips+1):m){
-  #            tmp <- as.numeric(res[[i]] < RM)
-  #            tmp <- prob2fitchCoding(tmp)
-  #            tmp <- fitchCoding2ambiguous(tmp)
-  #            res[[i]] <- tmp
-  #        }
-  #        browser()
-  #        res[1:ntips] <- data
-  #        attributes(res) <- att
-  #        return(res)
-  #    }
   fun <- function(X) {
     rs <- rowSums(X) # apply(X, 1, sum)
     X / rs
@@ -380,7 +303,7 @@ mpr <- function(tree, data, cost = NULL, return = "prob") {
   #    else res[1:ntips] <- data[1:ntips]
   attributes(res) <- att
   fun2 <- function(x) {
-    x <- p2dna(x) # prob2fitchCoding(x)
+    x <- p2dna(x)
     fitchCoding2ambiguous(x)
   }
   if (return != "prob") {
@@ -399,14 +322,11 @@ mpr <- function(tree, data, cost = NULL, return = "prob") {
 
 #' @rdname ancestral.pml
 #' @param site.pattern logical, plot i-th site pattern or i-th site
+#' @importFrom grDevices hcl.colors
 #' @export
 plotAnc <- function(tree, data, i = 1, site.pattern = TRUE, col = NULL,
                     cex.pie = par("cex"), pos = "bottomright", ...) {
   y <- subset(data, select = i, site.pattern = site.pattern)
-  #   args <- list(...)
-  #   CEX <- if ("cex" %in% names(args))
-  #       args$cex
-  #   else par("cex")
   CEX <- cex.pie
   xrad <- CEX * diff(par("usr")[1:2]) / 50
   levels <- attr(data, "levels")
@@ -422,7 +342,7 @@ plotAnc <- function(tree, data, i = 1, site.pattern = TRUE, col = NULL,
   xrad <- CEX * diff(lastPP$x.lim * 1.1) / 50
   par(new = TRUE)
   plot(tree, label.offset = 1.1 * xrad, plot = TRUE, ...)
-  if (is.null(col)) col <- rainbow(nc)
+  if (is.null(col)) col <-  hcl.colors(nc) #rainbow(nc)
   if (length(col) != nc) {
     warning("Length of color vector differs from number of levels!")
   }
@@ -432,5 +352,103 @@ plotAnc <- function(tree, data, i = 1, site.pattern = TRUE, col = NULL,
     piecol = col, col = "black", bg = "lightblue", horiz = FALSE,
     width = NULL, height = NULL, cex = cex.pie
   )
-  if (!is.null(pos)) legend(pos, levels, text.col = col)
+  if (!is.null(pos)) legend(pos, legend=levels, text.col = col)
 }
+
+#
+# ACCTRAN
+#
+
+
+acctran2 <- function(tree, data) {
+  tree <- reorder(tree, "postorder")
+  edge <- tree$edge
+  f <- init_fitch(data, FALSE, FALSE, m=2L)
+  psc_node <- f$pscore_node(edge)
+  tmp <- reorder(tree)$edge
+  tmp <- tmp[tmp[,2]>Ntip(tree),]
+  f$traverse(edge)
+  if(length(tmp)>0)f$acctran_traverse(tmp)
+  psc <- f$pscore_acctran(edge)
+  el <- psc #[edge[,2]]
+  parent <- unique(edge[,1])
+  desc <- Descendants(tree, parent, "children")
+  for(i in seq_along(parent)){
+    x <- psc_node[parent[i]] -sum(psc[desc[[i]]])
+    if(x>0) el[desc[[i]] ] <- el[desc[[i]] ] + x/length(desc[[i]])
+  }
+  tree$edge.length <- el[edge[,2]]
+  tree
+}
+
+
+#' @rdname parsimony
+#' @export
+acctran <- function(tree, data) {
+  if (inherits(tree, "multiPhylo")) {
+    compress <- FALSE
+    if (!is.null(attr(tree, "TipLabel"))){
+      compress <- TRUE
+      tree <- .uncompressTipLabel(tree)
+    }
+    res <- lapply(tree, acctran2, data)
+    class(res) <- "multiPhylo"
+    if (compress) res <- .compressTipLabel(res)
+    return(res)
+  }
+  acctran2(tree, data)
+}
+
+
+ptree <- function(tree, data, return = "prob") {
+  tree <- reorder(tree, "postorder")
+  edge <- tree$edge
+  att <- attributes(data)
+  nr <- att$nr
+  type <- att$type
+  m <- max(edge)
+  nTip <- Ntip(tree)
+  f <- init_fitch(data, FALSE, FALSE, m=2L)
+  f$traverse(edge)
+  tmp <- reorder(tree)$edge
+  tmp <- tmp[tmp[,2]>Ntip(tree),]
+  if(length(tmp)>0)f$acctran_traverse(tmp)
+  res <- vector("list", m)
+  att$names <- c(att$names, as.character((nTip+1):m))
+  if(return == "phyDat"){
+    res[1:nTip] <- data[1:nTip]
+    if(type=="DNA"){
+      for(i in (nTip+1):m)
+        res[[i]] <- f$getAnc(i)[1:nr]
+    }
+  }
+  if(return == "phyDat"){
+    res[1:nTip] <- data[1:nTip]
+    if(type=="DNA"){
+      for(i in (nTip+1):m)
+        res[[i]] <- f$getAncAmb(i)[1:nr]
+    }
+    else stop("This is only for nucleotide equences supported so far")
+  }
+  else {
+    fun <- function(X) {
+      rs <- rowSums(X) # apply(X, 1, sum)
+      X / rs
+    }
+    contrast <- att$contrast
+    for(i in seq_len(nTip))res[[i]] <- contrast[data[[i]], ]
+    for(i in (nTip+1):m)
+      res[[i]] <- f$getAnc(i)[1:nr, ]
+    res <- lapply(res, fun)
+  }
+  attributes(res) <- att
+  res
+}
+
+#parsimony.plot <- function(tree, ...) {
+#  x <- numeric(max(tree$edge))
+#  x[tree$edge[, 2]] <- tree$edge.length
+#  plot(tree, ...)
+#  ind <- get("last_plot.phylo", envir = .PlotPhyloEnv)$edge[, 2]
+#  edgelabels(prettyNum(x[ind]), frame = "none")
+#}
