@@ -1,10 +1,18 @@
 candidate.tree <- function(x){
-  tree <- random.addition(x)
-  tree <- optim.parsimony(tree, x)
-  tree <- multi2di(tree)
-  tree <- unroot(tree)
-  tree <- acctran(tree, x)
-  tree$edge.length <- tree$edge.length / sum(attr(x, "weight"))
+#  if(attr(x, "nc") > 31){
+#     dm <- dist.ml(x)
+#     tree <- fastme.bal(dm, nni = TRUE, spr = FALSE, tbr = FALSE)
+#     tree$edge.length[tree$edge.length<0] <- 1e-8
+#  }
+#  else{
+    tree <- random.addition(x)
+    tree <- optim.parsimony(tree, x, trace=0)
+    tree <- multi2di(tree)
+    tree <- unroot(tree)
+#    tree <- dist.ml(fit$data, bf=fit$bf, Q=fit$Q) %>% nnls.tree(tree=tree)
+    tree <- acctran(tree, x)
+    tree$edge.length <- tree$edge.length / sum(attr(x, "weight"))
+#  }
   tree
 }
 
@@ -13,8 +21,8 @@ candidate.tree <- function(x){
 #'
 #' \code{bootstrap.pml} performs (non-parametric) bootstrap analysis and
 #' \code{bootstrap.phyDat} produces a list of bootstrapped data sets.
-#' \code{plotBS} plots a phylogenetic tree with the with the bootstrap values
-#' assigned to the (internal) edges.
+#' \code{plotBS} plots a phylogenetic tree with the bootstrap values assigned
+#' to the (internal) edges.
 #'
 #' It is possible that the bootstrap is performed in parallel, with help of the
 #' multicore package. Unfortunately the multicore package does not work under
@@ -103,9 +111,13 @@ bootstrap.pml <- function(x, bs = 100, trees = TRUE, multicore = FALSE,
   extras <- match.call(expand.dots = FALSE)$...
   rearr <- c("optNni", "rearrangement")
   tmp <- pmatch(names(extras), rearr)
+  tmp <- tmp[!is.na(tmp)]
   do_rearr <- FALSE
-  if(!is.na(tmp)){
-    if(tmp==1) do_rearr <- extras$optNni
+  if(length(tmp)>0){
+    if(tmp==1){
+      do_rearr <- extras$optNni
+      if(is.name(do_rearr)) do_rearr <- as.logical(as.character(do_rearr))
+    }
     if(tmp==2) do_rearr <- extras$rearrangement %in% c("NNI", "stochastic",
                                                        "ratchet")
   }
@@ -204,7 +216,7 @@ bootstrap.phyDat <- function(x, FUN, bs = 100, multicore = FALSE,
       res <- lapply(BS, fitPar, x, ...)
     }
   }
-  if (class(res[[1]]) == "phylo") {
+  if (inherits(res[[1]], "phylo")) {
     class(res) <- "multiPhylo"
     res <- .compressTipLabel(res) # save memory
   }
@@ -323,6 +335,9 @@ plotBS <- function(tree, BStrees, type = "unrooted", bs.col = "black",
 #' If a list of partition is provided then the clade credibility is computed
 #' for the trees in x.
 #'
+#' \code{allCompat} returns a 50% majority rule consensus tree with added
+#' compatible splits similar to the option allcompat in MrBayes.
+#'
 #' @param x \code{x} is an object of class \code{multiPhylo} or \code{phylo}
 #' @param tree logical indicating whether return the tree with the clade
 #' credibility (default) or the clade credibility score for all trees.
@@ -348,11 +363,14 @@ plotBS <- function(tree, BStrees, type = "unrooted", bs.col = "black",
 #' majority_consensus <- consensus(bs, p=.5)
 #' all_compat <- allCompat(bs)
 #' max_clade_cred <- maxCladeCred(bs)
+#'
+#' old.par <- par(no.readonly = TRUE)
 #' par(mfrow = c(2,2), mar = c(1,4,1,1))
 #' plot(strict_consensus, main="Strict consensus tree")
 #' plot(majority_consensus, main="Majority consensus tree")
 #' plot(all_compat, main="Majority consensus tree with compatible splits")
 #' plot(max_clade_cred, main="Maximum clade credibility tree")
+#' par(old.par)
 #'
 #' # compute clade credibility for trees given a prop.part object
 #' pp <- prop.part(bs)
