@@ -48,8 +48,22 @@
 }
 
 
+wpgma_weights <- function(tree){
+  ntips <- Ntip(tree)
+  tree_u <- unroot(tree)
+  ind <- sort(unique( tree_u$edge[,1]) )[-1]
+  desc <- Descendants(tree_u, ind)
+  x <- seq_len(ntips)
+  w <- integer( ntips * (ntips-1L) / 2 )
+  for(i in seq_along(ind)){
+    ind <- getIndex(desc[[i]], x[-desc[[i]]], ntips)
+    w[ind] <- w[ind] + 1L
+  }
+  w
+}
+
 upgma.edge.length <- function(x, dm, method = "average") {
-  METHODS <- c("average", "single", "complete") # , "mcquitty"
+  METHODS <- c("average", "single", "complete", "mcquitty")
   i.meth <- match.arg(method, METHODS)
   X <- designTree(x, "rooted", TRUE)
   labels <- x$tip
@@ -62,10 +76,22 @@ upgma.edge.length <- function(x, dm, method = "average") {
   X@x[] <- 1
   nh <- numeric(max(x$edge))
   Y <- X * dm
+  if(i.meth=="mcquitty"){
+    w <- .5 ^ wpgma_weights(x)
+    Y <- X * (dm * w)
+    Y[,1] <- Y[,1]/2
+#    Y = as.matrix(X)
+#    beta = solve( t(Y) %*% diag(w) %*% Y ) %*% (t(Y) %*% diag(w) %*% y)
+#    nh[X@nodes] <- beta
+#    x$edge.length <- (nh[x$edge[, 1]] - nh[x$edge[, 2]]) / 2
+#    return( x )
+  }
+
   for (i in 1:(length(node) - 1)) {
     pos <- ind[(node[i] + 1):node[i + 1]]
     tmp <- switch(i.meth,
       average = mean(Y[pos, i]),
+      mcquitty = sum(Y[pos, i]),
       single = min(Y[pos, i]),
       complete = max(Y[pos, i]))
     nh[X@nodes[i]] <- tmp
@@ -77,7 +103,7 @@ upgma.edge.length <- function(x, dm, method = "average") {
 
 upgma_nni <- function(d, method = "average", opt = "min", trace = 0,
                       mc.cores = 2L){
-  METHODS <- c("average", "single", "complete") # , "mcquitty"
+  METHODS <- c("average", "single", "complete", "mcquitty")
   method <- match.arg(method, METHODS)
   OPT <- c("min", "ls")
   opt <- match.arg(opt, OPT)
@@ -97,11 +123,11 @@ upgma_nni <- function(d, method = "average", opt = "min", trace = 0,
     trees <- .uncompressTipLabel(trees)
     trees <- unclass(trees)
     nni.trees <- lapply(trees, upgma.edge.length, y, method = method)
-    ind <- which(sapply(nni.trees, function(x) !any(x$edge.length < 0)))
+    ind <- which(vapply(nni.trees, function(x) !any(x$edge.length < 0), FALSE))
     if (length(ind) == 0) return(best.tree)
     nni.trees <- nni.trees[ind]
     if (opt == "min") {
-      ME <- sapply(nni.trees, function(x) sum(x$edge.length))
+      ME <- vapply(nni.trees, function(x) sum(x$edge.length), 0)
       if (any(ME < bestME)) {
         bestME <- min(ME)
         count_nni <- count_nni + 1
@@ -111,7 +137,7 @@ upgma_nni <- function(d, method = "average", opt = "min", trace = 0,
       else run.nni <- FALSE
     }
     else {
-      LS <- sapply(nni.trees, function(x) sum( (coph(x) - y)^2))
+      LS <- vapply(nni.trees, function(x) sum( (coph(x) - y)^2), 0)
       if (any(LS < bestLS)) {
         bestLS <- min(LS)
         count_nni <- count_nni + 1
@@ -181,12 +207,13 @@ UNJ <- function(x){
   w <- rep(1, l)
   while (l > 2) {
     r <- rowSums(d) / (l - 2)
-    i <- 0
-    j <- 0
-    tmp <- .C("out", as.double(d), as.double(r), as.integer(l), as.integer(i),
-              as.integer(j))
-    e2 <- tmp[[5]]
-    e1 <- tmp[[4]]
+#    i <- 0
+#    j <- 0
+    tmp <- out_cpp(d, r, l)
+#    tmp <- .C("out", as.double(d), as.double(r), as.integer(l), as.integer(i),
+#              as.integer(j))
+    e2 <- tmp[2]
+    e1 <- tmp[1]
     l1 <- d[e1, e2] / 2 + sum( (d[e1, -c(e1, e2)] - d[e2, -c(e1, e2)]) *
                                 w[-c(e1, e2)]) / (2 * (n - w[e1] - w[e2]))
     l2 <- d[e1, e2] / 2 + sum( (d[e2, -c(e1, e2)] - d[e1, -c(e1, e2)]) *
@@ -214,77 +241,6 @@ UNJ <- function(x){
 }
 
 
-PNJ <- function(data)
-{
-  q <- l <- r <- length(data)
-  weight <- attr(data, "weight")
-
-  height <- NULL
-  parentNodes <- NULL
-  childNodes <- NULL
-  nam <- names(data)
-  tip.label <- nam
-  edge <- 1:q
-
-  z <- 0
-  D <- matrix(0, q, q)
-
-  for (i in 1:(l - 1)) {
-    for (j in (i + 1):l) {
-      w <- (data[[i]] * data[[j]]) %*% c(1, 1, 1, 1)
-      D[i, j] <- sum(weight[w == 0])
-    }
-  }
-
-  while (l > 1) {
-    l <- l - 1
-    z <- z + 1
-    d <- D + t(D)
-    if (l > 1) r <- rowSums(d) / (l - 1)
-    if (l == 1) r <- rowSums(d)
-    M <- d - outer(r, r, "+")
-    diag(M) <- Inf
-
-    e <- which.min(M)
-    e0 <- e %% length(r)
-    e1 <- ifelse(e0 == 0, length(r), e0)
-    e2 <- ifelse(e0 == 0, e %/% length(r), e %/% length(r) + 1)
-
-    ind <- c(e1, e2)
-    len <- d[e] / 2
-    nam <- c(nam[-ind], as.character(-l))
-
-    parentNodes <- c(parentNodes, -l, -l)
-    childNodes <- c(childNodes, edge[e1], edge[e2])
-
-    height <- c(height, len, len)
-    edge <- c(edge[-ind], -l)
-    w <- (data[[e1]] * data[[e2]]) %*% c(1, 1, 1, 1)
-    w <- which(w == 0)
-    newDat <- data[[e1]] * data[[e2]]
-    newDat[w, ] <- data[[e1]][w, ] + data[[e2]][w, ]
-    data <- data[-c(e1, e2)]
-    data[[l]] <- newDat
-    if (l > 1) {
-      D <- as.matrix(D[, -ind])
-      D <- D[-ind, ]
-      dv <- numeric(l - 1)
-      for (i in 1:(l - 1)) {
-        w <- (data[[i]] * data[[l]]) %*% c(1, 1, 1, 1)
-        dv[i] <- sum(weight[w == 0])
-      }
-      D <- cbind(D, dv)
-      D <- rbind(D, 0)
-    }
-  }
-  tree <- list(edge = cbind(as.character(parentNodes),
-                            as.character(childNodes)), tip.label = tip.label)
-  class(tree) <- "phylo"
-  tree <- old2new.phylo(tree)
-  reorder(tree)
-}
-
-
 #
 # Distance Matrix methods
 #
