@@ -85,10 +85,13 @@ sankoff.quartet <- function(dat, cost, p, l, weight) {
 #'
 #' @rdname parsimony
 #' @export
-parsimony <- function(tree, data, method = "fitch", ...) {
-  if (class(data)[1] != "phyDat") stop("data must be of class phyDat")
-  if (method == "sankoff") result <- sankoff(tree, data, ...)
-  if (method == "fitch") result <- fitch(tree, data, ...)
+## parsimony <- function(tree, data, cost=NULL, method = NULL)
+parsimony <- function(tree, data, method = "fitch", cost=NULL, site = "pscore"){
+  if (!inherits(data, "phyDat")) stop("data must be of class phyDat")
+  method <- match.arg(method, c("fitch", "sankoff"))
+  if(!any(is.binary(tree)) || !is.null(cost)) method <- "sankoff"
+  if (method == "sankoff") result <- sankoff(tree, data, cost=cost, site = site)
+  if (method == "fitch") result <- fitch(tree, data, site = site)
   result
 }
 
@@ -121,9 +124,7 @@ compressSites <- function(data) {
   l <- length(lev)
   nr <- attr(data, "nr")
   nc <- length(data)
-
   data <- unlist(data, FALSE, FALSE)
-
   attr(data, "dim") <- c(nr, nc)
   uni <- match(lev, LEV)
   fun <- function(x, uni) {
@@ -131,27 +132,33 @@ compressSites <- function(data) {
     res <- if (any(is.na(match(u, uni)))) return(x)
     match(x, u)
   }
-  data <- t(apply(data, 1, fun, uni))
-  ddd <- fast.table(data)
-  data <- ddd$data
-  class(data) <- "list"
-  attrData$weight <- tapply(attrData$weight, ddd$index, sum)
+  data <- apply(data, 1, fun, uni)
+  index <- grp_duplicated(data, MARGIN=2L)
+  pos <- which(!duplicated(index))
+  ntaxa <- nrow(data)
+  res <- vector("list", ntaxa)
+  for(i in seq_len(ntaxa)) res[[i]] <- data[i, pos]
+  attrData$weight <- tapply(attrData$weight, index, sum)
   attrData$index <- NULL
   attrData$nr <- length(attrData$weight)
   attrData$compressed <- TRUE
-  attributes(data) <- attrData
-  data
+  attributes(res) <- attrData
+  res
 }
 
 
-#
-# Branch and bound
-#
-
-parsinfo <- function(x) {
-  low <- lowerBound(x)
+parsinfo <- function(x, exact=TRUE) {
+  nstates <- attr(x, "nc")
   up <- upperBound(x)
-  ind <- which(low == up)
+  eps <- 1e-8
+  low <- up
+  low[up > (nstates - eps)] <- nstates - 1
+  if(exact){
+    ind <- which( (up > (1+eps))  & (up < (nstates-eps)) )
+    if(length(ind)>0) low[ind] <- lowerBound(getRows( x, ind ))
+    ind <- which(low == up)
+  }
+  else ind <- which(up < (1+eps) )
   cbind(ind, low[ind])
 }
 
@@ -164,20 +171,14 @@ lowerBound <- function(x, cost = NULL) {
   contrast <- attr(x, "contrast")
   rownames(contrast) <- attr(x, "allLevels")
   colnames(contrast) <- attr(x, "levels")
-  attr(x, "weight") <- rep(1, nr)
-  attr(x, "index") <- NULL
-
-  y <- as.character(x)
-  #    states <- apply(y, 2, unique.default) return type not known
-  states <- vector("list", ncol(y))
-  for (i in seq_len(ncol(y))) states[[i]] <- unique.default(y[, i])
-
-  singles <- which(rowSums(contrast) == 1) #
-  noinfo <- which(rowSums(contrast) == nc) #
+  nmax <- nrow(contrast)
+  z <- matrix(unlist(x, FALSE, FALSE), length(x), length(attr(x, "weight")),
+              byrow = TRUE)
+  states <- apply(z, 2, unique.default, nmax = nmax)
+  if(inherits(states, "matrix"))states <- asplit(states, 2)
+  singles <- which(rowSums(contrast) == 1)
+  noinfo <- which(rowSums(contrast) == nc)
   ambiguous <- which( (rowSums(contrast) > 1) & (rowSums(contrast) < nc))
-  singles <- names(singles)
-  noinfo <- names(noinfo)
-  ambiguous <- names(ambiguous)
 
   fun <- function(states, contrast, singles, noinfo, ambiguous) {
     if (length(states) == 1) return(0)
@@ -492,61 +493,45 @@ optim.parsimony <- function(tree, data, method = "fitch", cost = NULL,
 }
 
 
-## @param return return only best tree(s) or trees from each run.
 
 #' @rdname parsimony
 #' @export
-# perturbation="ratchet", "stochastic"
 pratchet <- function(data, start = NULL, method = "fitch", maxit = 1000,
-                     minit = 10, k = 10, trace = 1, all = FALSE,
-                     rearrangements = "SPR", perturbation = "ratchet", ...) {
-  search_history <- c(FALSE, FALSE)
-  # search_history <- c(FALSE, FALSE)
-  # return="single",
-  # if (return=="single") all <- FALSE
-  # if(return=="best") all <- TRUE
-  # if(return=="all" | return=="history"){
-  #  all <- TRUE
-  #  search_history <- c(TRUE, TRUE)
-  #}
-  # c("single", "best", "all", "history") needs better names
-
+                         minit = 10, k = 10, trace = 1, all = FALSE,
+                         rearrangements = "SPR", perturbation = "ratchet", ...) {
   eps <- 1e-08
-  # if(method=="fitch" && (is.null(attr(data, "compressed")) ||
-  #    attr(data, "compressed") == FALSE))
-  #      data <- compressSites(data)
   trace <- trace - 1
-  uniquetree <- function(trees) {
-    k <- 1
-    res <- trees[[1]]
-    result <- list()
-    result[[1]] <- res
-    k <- 2
-    trees <- trees[-1]
-    while (length(trees) > 0) {
-      class(trees) <- "multiPhylo"
-      rf <- suppressMessages( RF.dist(res, trees, FALSE) )
-      if (any(rf == 0)) trees <- trees[-which(rf == 0)]
-      if (length(trees) > 0) {
-        res <- trees[[1]]
-        result[[k]] <- res
-        k <- k + 1
-        trees <- trees[-1]
-      }
-    }
-    result
-  }
-  if (search_history[1]) start_trees <- list()
-  if (search_history[2]) search_trees <- list()
+
+  start_trees <- vector("list", maxit)
+  search_trees <- vector("list", maxit)
   tree <- NULL
   mp <- Inf
+
+  # remove parsimony uniformative sie or duplicates
+  if(perturbation == "ratchet"){
+    weight <- attr(data, "weight")
+    v <- rep(seq_along(weight), weight)
+    w <- logical(length(weight))
+  }
+  if(method=="fitch"){
+    data <- removeParsimonyUninfomativeSites(data, recursive=TRUE)
+    if(!is.null(attr(data, "informative"))) w[attr(data, "informative")] <- TRUE
+    else w[] <- TRUE
+  }
+  else data <- unique(data)
+
   if (perturbation != "random_addition"){
-    if(is.null(start)) start <- optim.parsimony(nj(dist.hamming(data)), data,
-                                        trace = trace, method = method,
+    if(is.null(start)) start <- optim.parsimony(fastme.ols(dist.hamming(data)),
+                                        data, trace = trace-1, method = method,
                                         rearrangements = rearrangements, ...)
     tree <- start
-    if(!is.binary(tree)) tree <- multi2di(tree)
-    data <- subset(data, tree$tip.label)
+    label <- intersect(tree$tip.label, names(data))
+    if (!is.binary(tree)){
+      tree <- multi2di(tree)
+      if(method=="fitch") tree <- unroot(tree)
+    }
+    data <- subset(data, label)
+    tree <- keep.tip(tree, label)
     attr(tree, "pscore") <- parsimony(tree, data, method = method, ...)
     mp <- attr(tree, "pscore")
     if (trace >= 0)
@@ -554,77 +539,82 @@ pratchet <- function(data, start = NULL, method = "fitch", maxit = 1000,
   }
   FUN <- function(data, tree, method, rearrangements, ...)
     optim.parsimony(tree, data = data, method = method,
-      rearrangements = rearrangements, ...)
-  result <- list()
-  result[[1]] <- tree
+                    rearrangements = rearrangements, ...)
+
+  result <- tree
   on.exit({
-    if (!all) result <- tree
-    else class(result) <- "multiPhylo"
+    if (!all && inherits(result, "multiPhylo")) result <- result[[1]]
+    if(!is.null(attr(data, "duplicated")))
+      result <- addTaxa(result, attr(data, "duplicated"))
+    #    else class(result) <- "multiPhylo"
     if (length(result) == 1) result <- result[[1]]
-#    if(return=="all"){
-#      all <- FALSE
-#      class(search_trees) <- "multiPhylo"
-#      search_trees <- .compressTipLabel(search_trees)
-#      result <- search_trees
-#    }
-#    if (return=="history"){
-#      class(start_trees) <- "multiPhylo"
-#      class(search_trees) <- "multiPhylo"
-#      start_trees <- .compressTipLabel(start_trees)
-#      search_trees <- .compressTipLabel(search_trees)
-#      result <- list(best = result,
-#                     start_trees = start_trees, search_trees = search_trees)
-#    }
+    env <- new.env()
+    start_trees <- start_trees[seq_len(i)]
+    search_trees <- search_trees[seq_len(i)]
+    class(start_trees) <- "multiPhylo"
+    class(search_trees) <- "multiPhylo"
+    start_trees <- .compressTipLabel(start_trees)
+    search_trees <- .compressTipLabel(search_trees)
+    assign("start_trees", start_trees, envir=env)
+    assign("search_trees", search_trees, envir=env)
+    if(perturbation == "ratchet") {
+      if(!is.null(attr(data, "duplicated")))
+        start_trees <- addTaxa(start_trees, attr(data, "duplicated"))
+      spl <- as.splits(start_trees)
+      result <- addConfidences(result, spl)
+      if (inherits(result, "multiPhylo")) result <- .compressTipLabel(result)
+    }
+    # for ratchet assign bs values
+    attr(result, "env") <- env
     return(result)
   })
   kmax <- 1
   nTips <- length(tree$tip.label)
   for (i in seq_len(maxit)) {
     if (perturbation == "ratchet") {
-      bstrees <- bootstrap.phyDat(data, FUN, tree = tree, bs = 1,
-        trace = trace, method = method, rearrangements = rearrangements, ...)
-      trees <- lapply(bstrees, optim.parsimony, data, trace = trace,
-        method = method, rearrangements = rearrangements, ...)
-      if (search_history[1]) start_trees[[i]] <- bstrees[[1]]
-      if (search_history[2]) search_trees[[i]] <- trees[[1]]
+      # sample and subset more efficient than in bootstrap.phyDat
+      bsw <- tabulate(sample(v, replace = TRUE), length(weight))[w]
+      bs_ind <- which(bsw > 0)
+      bs_data <- getRows(data, bs_ind)
+      attr(bs_data, "weight") <- bsw[bs_ind]
+      if(length(bs_ind) > 0)bstrees <- optim.parsimony(tree, bs_data,
+          trace = trace, method = method, rearrangements = rearrangements, ...)
+      else bstrees <- stree(length(data), tip.label = names(data))
+#      bstrees <- bootstrap.phyDat(data, FUN, tree = tree, bs = 1,
+#         trace = trace, method = method, rearrangements = rearrangements, ...)
+      trees <- optim.parsimony(bstrees, data, trace = trace,
+                        method = method, rearrangements = rearrangements, ...)
+      start_trees[[i]] <- bstrees
+      search_trees[[i]] <- trees
     }
     if (perturbation == "stochastic") {
       treeNNI <- rNNI(tree, floor(nTips / 2))
       trees <- optim.parsimony(treeNNI, data, trace = trace, method = method,
-        rearrangements = rearrangements, ...)
-      trees <- list(trees)
-      if (search_history[1]) start_trees[[i]] <- treeNNI
-      if (search_history[2]) search_trees[[i]] <- trees[[1]]
+                               rearrangements = rearrangements, ...)
+      start_trees[[i]] <- treeNNI
+      search_trees[[i]] <- trees
     }
     if (perturbation == "random_addition") {
       treeRA <- random.addition(data)
       trees <- optim.parsimony(treeRA, data, trace = trace, method = method,
-        rearrangements = rearrangements, ...)
-      trees <- list(trees)
-      if (search_history[1]) start_trees[[i]] <- treeRA
-      if (search_history[2]) search_trees[[i]] <- trees[[1]]
+                               rearrangements = rearrangements, ...)
+      start_trees[[i]] <- treeRA
+      search_trees[[i]] <- trees
     }
-    if (inherits(result, "phylo")) m <- 1
-    else m <- length(result)
-    if (m > 0) trees[2:(1 + m)] <- result[1:m]
-    pscores <- sapply(trees, function(data) attr(data, "pscore"))
+    pscores <- attr(trees, "pscore")
     mp1 <- min(pscores)
-    if ( (mp1 + eps) < mp) kmax <- 1
-    else kmax <- kmax + 1
-    mp <- mp1
-
+    if ( (mp1 + eps) < mp) {
+      kmax <- 1
+      result <- trees
+      mp <- mp1
+    }
+    else{
+      kmax <- kmax + 1
+      if( all && (mp1 < (mp + eps)) && all(RF.dist(trees, result) > 0))
+        result <- c(result, trees)
+    }
     if (trace >= 0)
       print(paste("Best pscore so far:", mp))
-    ind <- which(pscores < mp + eps)
-    if (length(ind) == 1) {
-      result <- trees[ind]
-      tree <- result[[1]]
-    }
-    else {
-      result <- uniquetree(trees[ind])
-      l <- length(result)
-      tree <- result[[sample(l, 1)]]
-    }
     if ( (kmax >= k) && (i >= minit)) break()
   } # for
 }  # pratchet
@@ -644,7 +634,7 @@ optim.sankoff <- function(tree, data, cost = NULL, trace = 1, ...) {
   if (is.rooted(tree)) tree <- unroot(tree)
   if (is.null(attr(tree, "order")) || attr(tree, "order") == "cladewise")
     tree <- reorder(tree, "postorder")
-  if (class(data)[1] != "phyDat") stop("data must be of class phyDat")
+  if (!inherits(data, "phyDat")) stop("data must be of class phyDat")
   addTaxa <- FALSE
   mapping <- map_duplicates(data)
   if (!is.null(mapping)) {
@@ -669,7 +659,7 @@ optim.sankoff <- function(tree, data, cost = NULL, trace = 1, ...) {
   pscore <- fit.sankoff(tree, dat, cost, "pscore")
 
   on.exit({
-    if (rt) tree <- ptree(tree, data)
+    if (rt) tree <- acctran(tree, data)
     if (addTaxa) {
       if (rt) tree <- add.tips(tree, tips = mapping[, 1], where = mapping[, 2],
           edge.length = rep(0, nrow(mapping)))
@@ -690,86 +680,3 @@ optim.sankoff <- function(tree, data, cost = NULL, trace = 1, ...) {
   if (trace > 0) cat("Final p-score", pscore, "after ", swap,
                      "nni operations \n")
 }
-
-
-#
-# ACCTRAN
-#
-ptree <- function(tree, data, type = "ACCTRAN", retData = FALSE) {
-  if (!inherits(data, "phyDat"))
-    stop("data must be of class phyDat")
-  if (is.null(attr(tree, "order")) || attr(tree, "order") ==
-    "cladewise")
-    tree <- reorder(tree, "postorder")
-  if (!is.binary(tree))
-    stop("Tree must be binary!")
-  tmp <- fitch(tree, data, site = "data")
-  nr <- attr(data, "nr")
-  node <- tree$edge[, 1]
-  edge <- tree$edge[, 2]
-  weight <- attr(data, "weight")
-#  m <- length(edge) + 1
-#  q <- length(tree$tip.label)
-  l <- as.integer(length(edge))
-  nTips <- length(tree$tip.label)
-  dat <- tmp[[2]]
-  if (!is.rooted(tree)) {
-    root <- getRoot(tree)
-    ind <- edge[node == root]
-    rSeq <- .C("fitchTriplet", integer(nr), dat[, ind[1]],
-      dat[, ind[2]], dat[, ind[3]], as.integer(nr))
-    dat[, root] <- rSeq[[1]]
-  }
-  result <- .C("ACCTRAN2", dat, as.integer(nr),
-    as.integer(node[l:1L]), as.integer(edge[l:1L]), l, as.integer(nTips))
-  el <- result[[5]][l:1L]
-  if (!is.rooted(tree)) {
-    ind2 <- which(node[] == root)
-    dat <- matrix(result[[1]], nr, max(node))
-    result <- .C("ACCTRAN3", result[[1]], as.integer(nr),
-      numeric(nr), as.integer(node[(l - 3L):1L]), as.integer(edge[(l -
-        3L):1L]), l - 3L, as.double(weight), numeric(l))
-    # , as.integer(nTips)
-    el <- result[[8]][(l - 3L):1L]
-    pars <- .C("fitchTripletACC4", dat[, root], dat[, ind[1]],
-      dat[, ind[2]], dat[, ind[3]], as.integer(nr), numeric(1),
-      numeric(1), numeric(1), as.double(weight), numeric(nr),
-      integer(nr))
-    el[ind2[1]] <- pars[[6]]
-    el[ind2[2]] <- pars[[7]]
-    el[ind2[3]] <- pars[[8]]
-  }
-  else {
-    result <- .C("ACCTRAN3", result[[1]], as.integer(nr),
-      numeric(nr), as.integer(node[l:1L]), as.integer(edge[l:1L]),
-      l, as.double(weight), numeric(l)) # , as.integer(nTips)
-    el <- result[[8]][l:1L]
-  }
-  tree$edge.length <- el
-  if (retData)
-    return(list(tree, matrix(result[[1]], nr, max(node))))
-  tree
-}
-
-
-#' @rdname parsimony
-#' @export
-acctran <- function(tree, data) {
-  if (inherits(tree, "multiPhylo")) {
-    compress <- FALSE
-    if (!is.null(attr(tree, "TipLabel"))) compress <- TRUE
-    res <- lapply(tree, ptree, data, type = "ACCTRAN", retData = FALSE)
-    class(res) <- "multiPhylo"
-    if (compress) res <- .compressTipLabel(res)
-    return(res)
-  }
-  ptree(tree, data, type = "ACCTRAN", retData = FALSE)
-}
-
-parsimony.plot <- function(tree, ...) {
-  x <- numeric(max(tree$edge))
-  x[tree$edge[, 2]] <- tree$edge.length
-  plot(tree, ...)
-  ind <- get("last_plot.phylo", envir = .PlotPhyloEnv)$edge[, 2]
-  edgelabels(prettyNum(x[ind]), frame = "none")
-}
