@@ -138,13 +138,13 @@ optCycle <- function(splits, tree) {
 
   M <- as.matrix(splits)
 
-  l <- as.integer(nrow(M))
+#  l <- as.integer(nrow(M))
   m <- as.integer(ncol(M))
 
   tmp <- tree$edge[, 2]
   tmp <- tmp[tmp <= m]
-
-  start <- .C("countCycle", M[, tmp], l, m, integer(1))[[4]]
+#  start <- .C("countCycle", M[, tmp], l, m, integer(1))[[4]]
+  start <- countCycle_cpp(M[, tmp])
   best <- start
   eps <- 1
   if (eps > 0) {
@@ -152,7 +152,8 @@ optCycle <- function(splits, tree) {
       tmptree <- rotate(tree, nodes[i])
       tmp <- tmptree$edge[, 2]
       tmp <- tmp[tmp <= m]
-      tmpC <- .C("countCycle", M[, tmp], l, m, integer(1))[[4]]
+#      tmpC <- .C("countCycle", M[, tmp], l, m, integer(1))[[4]]
+      tmpC <- countCycle_cpp(M[, tmp])
       if (tmpC < best) {
         best <- tmpC
         tree <- tmptree
@@ -166,10 +167,11 @@ optCycle <- function(splits, tree) {
 
 countCycles <- function(splits, tree = NULL, ord = NULL) {
   M <- as.matrix(splits)
-  l <- as.integer(nrow(M))
-  m <- as.integer(ncol(M))
+#  l <- as.integer(nrow(M))
+#  m <- as.integer(ncol(M))
   if (!is.null(tree)) ord  <- getOrdering(tree)
-  res <- .C("countCycle2", M[, ord], l, m, integer(l))[[4]]
+#  res <- .C("countCycle2", M[, ord], l, m, integer(l))[[4]]
+  res <- countCycle2_cpp(M[, ord])
   res
 }
 
@@ -242,7 +244,8 @@ as.splits.phylo <- function(x, ...) {
   }
   if (!is.null(x$node.label)) {
     conf <- x$node.label
-    if (is.character(conf)) conf <- as.numeric(conf)
+    if (is.character(conf)) as.is <- TRUE
+      #conf <- as.numeric(conf)
     if (!as.is) if (max(na.omit(conf)) > (1 + 1e-8)) conf <- conf / 100
     attr(result, "confidences") <- c(rep(NA_real_, length(x$tip.label)), conf)
   }
@@ -285,7 +288,7 @@ as.splits.prop.part <- function(x, ...) {
     attr(x, "weights") <- rep(1, length(x))
   else {
     attr(x, "weights") <- attr(x, "number")
-    attr(x, "confidences") <- attr(x, "number") / attr(x, "number")[1]
+    if( is.integer(attr(x, "number")) )attr(x, "confidences") <- attr(x, "number") / attr(x, "number")[1]
   }
   class(x) <- c("splits", "prop.part")
   x
@@ -307,6 +310,7 @@ as.splits.networx <- function(x, ...) {
 as.prop.part.splits <- function(x, ...) {
   attr(x, "number") <- attr(x, "weights")
   attr(x, "weights") <- NULL
+  attr(x, "confidences") <- NULL
   class(x) <- c("prop.part")
   x
 }
@@ -438,6 +442,14 @@ as.bitsplits.splits <- function(x) {
 }
 
 
+#' @rdname as.splits
+#' @method as.splits bitsplits
+#' @export
+as.splits.bitsplits <- function(x, ...){
+  as.splits(as.prop.part(x))
+}
+
+
 # computes compatible splits
 #' @rdname as.splits
 #' @export
@@ -470,6 +482,7 @@ compatible <- function(obj) {
 }
 
 
+# replace compatible ??
 compatible2 <- function(obj1, obj2 = NULL) {
   if (!inherits(obj1, "splits"))
     stop("obj needs to be of class splits")
